<?php
if (!defined('ABSPATH')) {
    die(__("Can't load this file directly", "woo-import-export"));
}

class WPIE_INIT {

    public function __construct() {

        if (is_admin()) {

            add_action('admin_notices', array(&$this, 'wpie_admin_notices'), 10099);

            add_filter('mod_rewrite_rules', array($this, 'wpie_protect_data'));

            add_filter('robots_txt', array($this, 'wpie_disallow_file_indexing'), 10, 2);

            add_action('admin_init', array($this, 'wpie_check_htaccess_updated'));

            add_action('admin_init', array(&$this, 'wpie_db_check'));

            add_action('admin_init', array(&$this, 'update_notices'));

            add_action('admin_enqueue_scripts', array(&$this, 'wpie_set_admin_css'), 10);

            add_action('admin_enqueue_scripts', array(&$this, 'wpie_set_admin_js'), 10);

            add_action('admin_head', array($this, 'wpie_hide_all_notice_to_admin_side'), 10000);

            add_filter('admin_footer_text', array(&$this, 'wpie_replace_footer_admin'));

            add_filter('update_footer', array(&$this, 'wpie_replace_footer_version'), 1234);

            add_action('admin_init', array(&$this, 'wpie_download_exported_data'));

            add_action('wp_ajax_wpie_activate_license', array(&$this, 'wpie_activate_license'));

            add_action('wp_ajax_wpie_license_deacivation', array(&$this, 'wpie_license_deacivation'));

            add_filter('wpmu_drop_tables', array(__CLASS__, 'wpmu_drop_tables'));

            $plugins = get_option('active_plugins');

            if (!function_exists('is_plugin_active_for_network')) {
                require_once(ABSPATH . 'wp-admin/includes/plugin.php');
            }

            $required_woo_plugin = 'woocommerce/woocommerce.php';

            if (in_array($required_woo_plugin, $plugins) || is_plugin_active_for_network($required_woo_plugin)) {

                if (class_exists('Woocommerce')) {
                    $this->wpie_init_plugin();
                } else {
                    add_action('woocommerce_loaded', array(&$this, 'wpie_init_plugin'));
                }
            }
            unset($plugins, $required_woo_plugin);
        }

        add_action('plugins_loaded', array(&$this, 'wpie_load_textdomain'));
    }

    /**
     * Uninstall tables when MU blog is deleted.
     *
     * @param  array $tables List of tables that will be deleted by WP.
     * @return array
     */
    public static function wpmu_drop_tables($tables = array()) {

        global $wpdb;

        $tables[] = $wpdb->prefix . 'wpie_export_log';

        return $tables;
    }

    public function update_notices() {

        if (isset($_GET['wpie-action']) && sanitize_text_field($_GET['wpie-action']) == "wpie-hide-notice" && isset($_GET['_nonce']) && !empty($_GET['_nonce'])) {

            $is_validate = wp_verify_nonce(sanitize_text_field($_GET['_nonce']), "wpie_security");

            if ($is_validate !== false) {

                update_user_meta(get_current_user_id(), "dismissed_wpie_file_security_notice", 1);
            }
        }
    }

    public function wpie_disallow_file_indexing($output = "", $public = true) {

        $output .= "User-agent: *\n";

        $output .= "Disallow: /wp-content/uploads/woo-import-export\n";

        return $output;
    }

    public function wpie_init_plugin() {

        if (is_admin()) {

            add_action('admin_menu', array(&$this, 'wpie_set_menu'));
        }
    }

    public function wpie_protect_data($rules = "") {

        $newRule = "RewriteCond %{REQUEST_FILENAME} -s" . PHP_EOL;

        $newRule .= "RewriteCond %{HTTP_USER_AGENT} !facebookexternalhit/[0-9]" . PHP_EOL;

        $newRule .= "RewriteCond %{HTTP_USER_AGENT} !Twitterbot/[0-9]" . PHP_EOL;

        $newRule .= "RewriteCond %{HTTP_USER_AGENT} !Googlebot/[0-9]" . PHP_EOL;

        $upload_dir_url = str_replace("https", "http", WPIE_UPLOAD_URL);

        $site_url = str_replace("https", "http", site_url());

        $newRule .= "RewriteRule " . str_replace(trailingslashit($site_url), '', $upload_dir_url) . "(\/[A-Za-z0-9_@.\/&+-]+)+\.([A-Za-z0-9_@.\/&+-]+)$ [L]" . PHP_EOL;

        update_option('wpie_is_admin_notice_clear', 1);

        unset($upload_dir_url, $site_url);

        return $newRule . $rules . PHP_EOL;
    }

    private function is_htaccess_writable() {

        $htaccess_file = get_home_path() . '.htaccess';

        if (!file_exists($htaccess_file)) {

            return false;
        }

        if (wp_is_writable($htaccess_file)) {
            return true;
        }

        @chmod($htaccess_file, 0666);

        if (!wp_is_writable($htaccess_file)) {
            return false;
        }

        unset($htaccess_file);

        return true;
    }

    public function wpie_check_htaccess_updated() {

        $is_updated = get_option('wpie_is_updated_file_security', false);

        if ($is_updated === false) {

            //update robots.txt
            $robots_file = get_home_path() . 'robots.txt';

            ob_start();

            do_robots();

            $robots_content = ob_get_clean();

            $fp = fopen($robots_file, 'a');

            fwrite($fp, $robots_content);

            fclose($fp);

            //update htaccess
            $is_htaccess_writable = $this->is_htaccess_writable();

            if ($is_htaccess_writable === true) {
                flush_rewrite_rules();
            }

            update_option('wpie_is_updated_file_security', 1);

            //unset all variables & clear memory
            unset($robots_file, $is_htaccess_writable);
        }

        unset($is_updated);
    }

    private function wpie_add_plugin_data() {

        $wpie_plugin_version = get_option('wpie_plugin_version');

        if ($wpie_plugin_version === false || $wpie_plugin_version == '') {

            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

            global $wpdb, $wpie_product, $wpie_order, $wpie_user, $wpie_product_category, $wpie_coupon;

            $charset_collate = '';

            if ($wpdb->has_cap('collation')) {

                if (!empty($wpdb->charset))
                    $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";

                if (!empty($wpdb->collate))
                    $charset_collate .= " COLLATE $wpdb->collate";
            }

            update_option('wpie_plugin_version', WPIE_PLUGIN_VERSION);

            $export_log = $wpdb->prefix . 'wpie_export_log';

            $export_log_table = "CREATE TABLE IF NOT EXISTS {$export_log}(
							
                            export_log_id INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY, 
                            export_log_file_type VARCHAR(150) NOT NULL, 
                            export_log_file_name VARCHAR(150) NOT NULL,
                            export_log_data VARCHAR(150) NOT NULL,
                            create_date DATETIME NOT NULL 

                            ){$charset_collate}";

            dbDelta($export_log_table);

            $product_fields = $wpie_product->get_new_product_fields();

            update_option('wpie_product_fields', $product_fields);

            $order_fields = $wpie_order->get_new_order_fields();

            update_option('wpie_order_fields', $order_fields);

            $user_fields = $wpie_user->get_new_user_fields();

            update_option('wpie_user_fields', $user_fields);

            $product_cat_fields = $wpie_product_category->get_new_product_cat_fields();

            update_option('wpie_product_cat_fields', $product_cat_fields);

            $coupon_fields = $wpie_coupon->get_new_coupon_fields();

            update_option('wpie_coupon_fields', $coupon_fields);

            unset($charset_collate, $export_log, $export_log_table, $product_fields, $order_fields, $user_fields, $product_cat_fields, $coupon_fields);
        }

        unset($wpie_plugin_version);
    }

    public function wpie_db_check() {

        self::wpie_set_time_limit(0);

        self::wpie_add_plugin_data();
    }

    public function wpie_load_textdomain() {
        load_plugin_textdomain('woo-import-export', false, 'woo-import-export/languages/');
    }

    public function wpie_set_admin_css() {

        wp_enqueue_style('wpie-global-admin-css', WPIE_CSS_URL . '/wpie-global-admin.min.css', array(), WPIE_PLUGIN_VERSION);

        if (isset($_GET['page'])) {

            wp_register_style('wpie-admin-css', WPIE_CSS_URL . '/wpie-admin.min.css', array(), WPIE_PLUGIN_VERSION);

            if ($_GET['page'] == 'wpie-home' || $_GET['page'] == 'wpie-products' || $_GET['page'] == 'wpie-product-categories' || $_GET['page'] == 'wpie-orders' || $_GET['page'] == 'wpie-users' || $_GET['page'] == 'wpie-coupons' || $_GET['page'] == 'wpie-settings') {

                wp_enqueue_style('wpie-admin-css');

                wp_enqueue_style('bootstrap-css', WPIE_CSS_URL . '/bootstrap.min.css');
            }

            if ($_GET['page'] == 'wpie-products' || $_GET['page'] == 'wpie-orders' || $_GET['page'] == 'wpie-coupons' || $_GET['page'] == 'wpie-users') {

                wp_enqueue_style('bootstrap-datepicker-css', WPIE_CSS_URL . '/bootstrap-datepicker.min.css');
            }

            if ($_GET['page'] == 'wpie-products' || $_GET['page'] == 'wpie-product-categories' || $_GET['page'] == 'wpie-orders' || $_GET['page'] == 'wpie-coupons' || $_GET['page'] == 'wpie-users') {

                wp_enqueue_style('bootstrap-css', WPIE_CSS_URL . '/bootstrap.min.css');

                wp_enqueue_style('datatables-css', WPIE_CSS_URL . '/datatables.min.css');

                wp_enqueue_style('animate-css', WPIE_CSS_URL . '/animate.css');

                wp_enqueue_style('chosen-css', WPIE_CSS_URL . '/chosen.min.css');

                wp_enqueue_style('datatables-bootstrap-css', WPIE_CSS_URL . '/datatables-bootstrap.min.css');
            }
        }
    }

    public function wpie_set_admin_js() {

        if (isset($_GET['page'])) {

            wp_register_script('wpie-admin-js', WPIE_JS_URL . '/wpie-admin.min.js', array('jquery'), WPIE_PLUGIN_VERSION, true);

            if ($_GET['page'] == 'wpie-home' || $_GET['page'] == 'wpie-products' || $_GET['page'] == 'wpie-product-categories' || $_GET['page'] == 'wpie-orders' || $_GET['page'] == 'wpie-users' || $_GET['page'] == 'wpie-coupons' || $_GET['page'] == 'wpie-settings') {

                wp_enqueue_script('jquery');

                wp_enqueue_script('plupload');

                wp_enqueue_script('wpie-admin-js');

                $wpie_localize_script_data = array(
                    'ajaxURL' => admin_url('admin-ajax.php'),
                    'wpie_site_url' => site_url(),
                    'upload_url' => WPIE_UPLOAD_URL,
                    'upload_dir' => WPIE_UPLOAD_DIR,
                    'plugin_url' => WPIE_PLUGIN_URL,
                    'localizeText' => $this->wpie_load_msg()
                );

                wp_localize_script('wpie-admin-js', 'wpiePluginSettings', $wpie_localize_script_data);

                unset($wpie_localize_script_data);

                wp_enqueue_script('bootstrap-js', WPIE_JS_URL . '/bootstrap.min.js', array('jquery'), WPIE_PLUGIN_VERSION, true);

                wp_enqueue_script('bootstrap-notify-js', WPIE_JS_URL . '/bootstrap-notify.min.js', array('jquery', 'bootstrap-js'), WPIE_PLUGIN_VERSION, true);

                wp_enqueue_script('bpopup-js', WPIE_JS_URL . '/bpopup.min.js', array('jquery'), WPIE_PLUGIN_VERSION, true);
            }
            if ($_GET['page'] == 'wpie-products' || $_GET['page'] == 'wpie-orders' || $_GET['page'] == 'wpie-coupons' || $_GET['page'] == 'wpie-users') {

                wp_enqueue_script('bootstrap-datepicker-js', WPIE_JS_URL . '/bootstrap-datepicker.min.js', array('jquery'), WPIE_PLUGIN_VERSION, true);
            }
            if ($_GET['page'] == 'wpie-products' || $_GET['page'] == 'wpie-product-categories' || $_GET['page'] == 'wpie-orders' || $_GET['page'] == 'wpie-coupons' || $_GET['page'] == 'wpie-users') {

                wp_enqueue_script('chosen-js', WPIE_JS_URL . '/chosen.min.js', array('jquery'), WPIE_PLUGIN_VERSION, true);

                wp_enqueue_script('datatables-js', WPIE_JS_URL . '/datatables.min.js', array('jquery'), WPIE_PLUGIN_VERSION, true);

                wp_enqueue_script('plupload-all');
            }
        }
    }

    public function wpie_hide_all_notice_to_admin_side() {
        if (isset($_GET['page']) && ($_GET['page'] == 'wpie-home' || $_GET['page'] == 'wpie-products' || $_GET['page'] == 'wpie-product-categories' || $_GET['page'] == 'wpie-orders' || $_GET['page'] == 'wpie-users' || $_GET['page'] == 'wpie-coupons' || $_GET['page'] == 'wpie-settings')) {
            remove_all_actions('admin_notices', 10000);
            remove_all_actions('all_admin_notices', 10000);
            remove_all_actions('network_admin_notices', 10000);
            remove_all_actions('user_admin_notices', 10000);
        }
    }

    public function wpie_admin_notices() {

        if (get_option('wpie_is_admin_notice_clear', false) === false) {

            $notice = get_user_meta(get_current_user_id(), "dismissed_wpie_file_security_notice", true);

            if (intval($notice) != 1) {
                ?>
                <div class="wpie-message updated" >
                    <a class="wpie-message-close notice-dismiss" href="<?php echo esc_url(wp_nonce_url(add_query_arg("wpie-action", 'wpie-hide-notice'), 'wpie_security', '_nonce')); ?>"><?php _e('Dismiss', 'woo-import-export'); ?></a>
                    <p><b><?php echo __('WP Import Export : ', 'woo-import-export'); ?></b> <?php _e("If your <b>.htaccess</b> file were writable, we could do this automatically, but it isn’t. So you must either make it writable or manually update your .htaccess with the mod_rewrite rules found under <b>Settings >> Permalinks</b>. Until then, the exported and imported files are not protected from direct access.", 'woo-import-export'); ?></p>
                </div>
                <?php
            }
        }
    }

    public function wpie_set_menu() {

        global $current_user;

        if (current_user_can('administrator') || is_super_admin()) {

            $wpie_caps = $this->wpie_user_capabilities();

            foreach ($wpie_caps as $wpie_cap => $cap_desc) {
                $current_user->add_cap($wpie_cap);
            }

            unset($wpie_caps);
        }

        $menu_place = $this->get_dynamic_position(28.1, .1);

        add_menu_page(__('WooCommerce Import Export Dashboard', 'woo-import-export'), __('Woo Imp Exp', 'woo-import-export'), 'wpie_manage_dashboard', 'wpie-home', array(&$this, 'wpie_get_page'), WPIE_IMAGES_URL . '/wpie_logo.png', (string) $menu_place);

        add_submenu_page('wpie-home', __('Dashboard', 'woo-import-export'), __('Dashboard', 'woo-import-export'), 'wpie_manage_dashboard', 'wpie-home', array(&$this, 'wpie_get_page'));

        add_submenu_page('wpie-home', __('Products', 'woo-import-export'), __('Products', 'woo-import-export'), 'wpie_manage_products', 'wpie-products', array(&$this, 'wpie_get_page'));

        add_submenu_page('wpie-home', __('Product Categories', 'woo-import-export'), __('Product Categories', 'woo-import-export'), 'wpie_manage_product_categories', 'wpie-product-categories', array(&$this, 'wpie_get_page'));

        add_submenu_page('wpie-home', __('Orders', 'woo-import-export'), __('Orders', 'woo-import-export'), 'wpie_manage_orders', 'wpie-orders', array(&$this, 'wpie_get_page'));

        add_submenu_page('wpie-home', __('Users', 'woo-import-export'), __('Users', 'woo-import-export'), 'wpie_manage_users', 'wpie-users', array(&$this, 'wpie_get_page'));

        add_submenu_page('wpie-home', __('Coupons', 'woo-import-export'), __('Coupons', 'woo-import-export'), 'wpie_manage_coupons', 'wpie-coupons', array(&$this, 'wpie_get_page'));

        unset($menu_place);
    }

    public function wpie_get_page() {

        if (isset($_GET['page'])) {

            if ($_GET['page'] == 'wpie-home' && file_exists(WPIE_VIEW_DIR . '/wpie-dashboard.php')) {

                require_once( WPIE_VIEW_DIR . '/wpie-dashboard.php');
            } else if ($_GET['page'] == 'wpie-home' && file_exists(WPIE_VIEW_DIR . '/wpie-dashboard.php')) {

                require_once( WPIE_VIEW_DIR . '/wpie-dashboard.php');
            } else if ($_GET['page'] == 'wpie-products' && file_exists(WPIE_VIEW_DIR . '/wpie-products.php')) {

                require_once( WPIE_VIEW_DIR . '/wpie-products.php');
            } else if ($_GET['page'] == 'wpie-product-categories' && file_exists(WPIE_VIEW_DIR . '/wpie-product-categories.php')) {

                require_once( WPIE_VIEW_DIR . '/wpie-product-categories.php');
            } else if ($_GET['page'] == 'wpie-orders' && file_exists(WPIE_VIEW_DIR . '/wpie-orders.php')) {

                require_once( WPIE_VIEW_DIR . '/wpie-orders.php');
            } else if ($_GET['page'] == 'wpie-users' && file_exists(WPIE_VIEW_DIR . '/wpie-users.php')) {

                require_once( WPIE_VIEW_DIR . '/wpie-users.php');
            } else if ($_GET['page'] == 'wpie-coupons' && file_exists(WPIE_VIEW_DIR . '/wpie-coupons.php')) {

                require_once( WPIE_VIEW_DIR . '/wpie-coupons.php');
            }
        }
    }

    private function wpie_user_capabilities() {

        return array(
            'wpie_manage_dashboard' => __('User can manage Woo Imp Exp Dashboard', 'woo-import-export'),
            'wpie_manage_products' => __('User can manage Products', 'woo-import-export'),
            'wpie_manage_orders' => __('User can manage Orders', 'woo-import-export'),
            'wpie_manage_users' => __('User can manage Users', 'woo-import-export'),
            'wpie_manage_product_categories' => __('User can manage Product Categories', 'woo-import-export'),
            'wpie_manage_coupons' => __('User can manage Coupons', 'woo-import-export'),
        );
    }

    private function get_dynamic_position($start, $increment = 0.1) {

        if (isset($GLOBALS['menu']) && !empty($GLOBALS['menu'])) {

            $menus_positions = array();

            foreach ($GLOBALS['menu'] as $key => $menu) {
                $menus_positions[] = $key;
            }

            if (!in_array($start, $menus_positions)) {
                return $start;
            }

            while (in_array($start, $menus_positions)) {
                $start += $increment;
            }

            unset($menus_positions);
        }

        return $start;
    }

    public function wpie_replace_footer_admin() {
        echo '';
    }

    public function wpie_replace_footer_version() {
        return '';
    }

    public function wpie_download_exported_data() {

        if (isset($_POST['wpie-product-csv-file-name']) && !empty($_POST['wpie-product-csv-file-name']) && file_exists(WPIE_UPLOAD_DIR . '/' . $_POST['wpie-product-csv-file-name'])) {

            $fileName = sanitize_file_name($_POST['wpie-product-csv-file-name']);

            header('Cache-Control:must-revalidate,post-check=0,pre-check=0');

            header('Content-Description: File Transfer');

            header('Content-Type: text/csv;');

            header('Content-Disposition: attachment; filename=' . $fileName);

            header('Expires:0');

            header('Pragma: public');

            readfile(WPIE_UPLOAD_DIR . '/' . $fileName);

            die();
        } else if (isset($_POST['wpie_download_exported_file']) && !empty($_POST['wpie_download_exported_file']) && !isset($_POST['action'])) {

            $log_id = absint(sanitize_text_field($_POST['wpie_download_exported_file']));

            if ($log_id > 0) {

                global $wpdb;

                $filename = $wpdb->get_var($wpdb->prepare("SELECT `export_log_file_name` FROM " . $wpdb->prefix . "wpie_export_log where `export_log_id` = %d", $log_id));

                if (file_exists(WPIE_UPLOAD_DIR . '/' . $filename)) {

                    header('Cache-Control:must-revalidate,post-check=0,pre-check=0');

                    header('Content-Description: File Transfer');

                    header('Content-Type: text/csv;');

                    header('Content-Disposition: attachment; filename=' . $filename);

                    header('Expires:0');

                    header('Pragma: public');

                    readfile(WPIE_UPLOAD_DIR . '/' . $filename);
                }

                die();
            }

            unset($log_id);
        }
    }

    public function wpie_activate_license() {

        $site_data = array();

        $return_value = array();

        $return_value['message'] = 'error';

        $site_data['customer_name'] = '';

        $site_data['customer_email'] = get_option('admin_email');

        $site_data['purchase_code'] = trim(sanitize_text_field($_POST["purchase_code"]));

        $site_data['domain_name'] = get_option('siteurl');

        $site_info = array();

        $site_info['blog_name'] = get_bloginfo('name');

        $site_info['description'] = get_bloginfo('description');

        $site_info['site_home_url'] = home_url();

        $site_info['admin_email'] = get_bloginfo('admin_email');

        $site_info['server_addr'] = $_SERVER['SERVER_ADDR'];

        $new_str = implode("^|^", $site_info);

        $new_plugin_code = base64_encode($new_str);

        $site_data['plugin_data'] = $new_plugin_code;

        $site_data['plugin_name'] = 'woo-import-export';

        $site_data['plugin_url'] = WPIE_PLUGIN_URL;

        $site_data['plugin_version'] = WPIE_PLUGIN_VERSION;

        $site_data['plugin_status'] = 'active';

        $valstring = maybe_serialize($site_data);

        $post_data = base64_encode($valstring);

        $response = wp_remote_post(WPIE_PLUGIN_SITE, array(
            'method' => 'POST',
            'timeout' => 45,
            'redirection' => 5,
            'httpversion' => '1.1',
            'blocking' => true,
            'headers' => array(),
            'body' => array(
                'verify_product_purchase' => $post_data),
            'cookies' => array()
                )
        );

        unset($site_data, $site_info, $new_str, $new_plugin_code, $valstring, $post_data);

        if (array_key_exists('body', $response) && isset($response["body"]) && $response["body"] != "") {

            $response_result = maybe_unserialize(base64_decode($response["body"]));

            if (isset($response_result{'vjit_verify'}) && $response_result{'vjit_verify'} == 'success') {

                update_option('wpie_current_site_plugin_date_format', isset($response_result{'vjit_verify_data'}) ? $response_result{'vjit_verify_data'} : 1);

                $return_value['message'] = $response_result{'vjit_verify'};

                $return_value['message_note'] = $response_result['vjit_verify_note'];

                $return_value['message_content'] = $response_result{'vjit_verify_msg'};
            } else {
                $return_value['message_content'] = __('Invalid Request Data.', 'woo-import-export');
            }
            unset($response_result);
        }
        unset($response);

        echo json_encode($return_value);

        die();
    }

    public function wpie_license_deacivation() {

        $site_data = array();

        $return_value = array();

        $return_value['message'] = 'error';

        $site_info = array();

        $site_info['blog_name'] = get_bloginfo('name');

        $site_info['description'] = get_bloginfo('description');

        $site_info['site_home_url'] = home_url();

        $site_info['admin_email'] = get_bloginfo('admin_email');

        $site_info['server_addr'] = $_SERVER['SERVER_ADDR'];

        $new_str = implode("^|^", $site_info);

        $wpie_date_format = get_option('wpie_current_site_plugin_date_format');

        if ($wpie_date_format && $wpie_date_format != "") {
            $plugin_data = maybe_unserialize(base64_decode($wpie_date_format));
        } else {
            $plugin_data = "";
        }

        $new_plugin_code = base64_encode($new_str);

        $site_data['plugin_info'] = $plugin_data;

        $site_data['plugin_data'] = $new_plugin_code;

        $site_data['plugin_name'] = 'woo-import-export';

        $site_data['plugin_url'] = WPIE_PLUGIN_URL;

        $site_data['plugin_version'] = WPIE_PLUGIN_VERSION;

        $site_data['plugin_status'] = 'deactive';

        $valstring = maybe_serialize($site_data);

        $post_data = base64_encode($valstring);

        $response = wp_remote_post(WPIE_PLUGIN_SITE, array(
            'method' => 'POST',
            'timeout' => 45,
            'redirection' => 5,
            'httpversion' => '1.1',
            'blocking' => true,
            'headers' => array(),
            'body' => array(
                'vjit_deacivate_license' => $post_data),
            'cookies' => array()
                )
        );

        unset($site_data, $site_info, $new_str, $wpie_date_format, $new_plugin_code, $valstring, $post_data);

        if (array_key_exists('body', $response) && isset($response["body"]) && $response["body"] != "") {

            $response_result = maybe_unserialize(base64_decode($response["body"]));

            if ($response_result{'vjit_verify'} == 'success') {
                $return_value['message'] = $response_result{'vjit_verify'};

                $return_value['message_note'] = $response_result['vjit_verify_note'];

                $return_value['message_content'] = $response_result{'vjit_verify_msg'};

                delete_option('wpie_current_site_plugin_date_format');
            } else {
                $return_value['message_content'] = __('Invalid Request Data.', 'woo-import-export');
            }
            unset($response_result);
        }

        unset($response);

        echo json_encode($return_value);

        die();
    }

    public function wpie_set_time_limit($time = 0) {

        $safe_mode = ini_get('safe_mode');

        if ((!$safe_mode) || strtolower($safe_mode) == 'off') {

            set_time_limit($time);

            ini_set("memory_limit", "-1");

            ini_set("auto_detect_line_endings", "1");
        }

        unset($safe_mode);
    }

    private function wpie_load_msg() {
        return array(
            "generatingCsvFile" => __('Generating CSV File', 'woo-import-export'),
            "emptyEmailRecipient" => __('Please Enter Email Recipient(s).', 'woo-import-export'),
            "emptyEmailSubject" => __('Please Enter Email Subject.', 'woo-import-export'),
            "emptyEmailBody" => __('Please Enter Email Body.', 'woo-import-export'),
            "invalidEmailAddress" => __('Please Enter valid Email Address.', 'woo-import-export'),
            "confirmDeleteScheduled" => __('Are you sure you want to delete this Scheduled ?', 'woo-import-export'),
            "confirmDeleteRecord" => __('Are you sure you want to delete this Record ?', 'woo-import-export'),
            "emptyFilename" => __('Please Enter Filename', 'woo-import-export'),
            "emptyPurchaseCode" => __('Please Enter Purchase Code', 'woo-import-export'),
            "ErrorWhileActivateLicense" => __('Error While Activate License', 'woo-import-export'),
            "filteringRecords" => __('Filtering Records', 'woo-import-export'),
            "noRecordsFound" => __('No Records Found', 'woo-import-export')
        );
    }

}
